/* m18st05b serial VFD
 * 
 *    example for AVR ATMega8(L)
 * 
 * UART code comes from mikrocontroller.net wiki
 * 
 */

#ifndef F_CPU
#define F_CPU 10000000	// my devboard uses a 10MHz XTAL
#endif

#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>

#include "vfd.h"

#define BAUDRATE 9600UL

void delay(uint16_t ms);
void uart_init(void);
void uart_putc(char c);
void uart_puts(char *str);

int main(void) {
	uart_init();
	
	/* display needs a few seconds to boot up when power is applied
	 * first time, 5s always works
	 * when using a system timer a simple "one shot" logic in timer
	 * ISR would prevent bootup delay.
	 */
	delay(5000);

	/* - clear display and set line break when EOL
	 *   scrolling instead of line break is bootup default, cmd unknown! 
	 * - dim level 5 (full on)
	 * - show red record button on display
	 * 
	 * all command sequences are stored in FLASH, NO SRAM usage.
	 */ 
	vfd_cmd_P(PSTR(VFD_CLEAR_AND_LBREAK));
	vfd_cmd_P(PSTR(VFD_DIM5));
	vfd_cmd_P(PSTR(VFD_SYMBOL_REC));
	
	// print string from FLASH
	vfd_puts_P(PSTR("## Futaba VFD #####   16x2   ###"));

	delay(3500);
	
	// print string from SRAM
	vfd_puts("String from SRAM....-.-.-##+~!@@");
	
	delay(3500);

	/* CLOCK SETUP and ENABLE 
	 * 
	 * This clock is running on display's MCU and once set but hidden
	 * by printing further text on display, it still runs in background
	 * and can be brought back by VFD_CLOCK_SHOW command.
	 * */
	vfd_cmd_P(PSTR(VFD_CLOCK_SHOW));
	vfd_cmd_P(PSTR(VFD_CLOCK_MOVING));	
	
	/* clock set command is sent from RAM instead of flash as this
	 * is subject to change once a minute ;)
	 *
	 *  this sets clock to 11:12 on 17th November 2019
	 */
	vfd_cmd("\x08\x1B\x00\x12\x11\x17\x11\x20\x19");

	/* frames below text around symbols */
	vfd_cmd_P(PSTR(VFD_FRAME_MEDIA));
	vfd_cmd_P(PSTR(VFD_FRAME_REC));
	vfd_cmd_P(PSTR(VFD_FRAME_SOURCE));
	vfd_cmd_P(PSTR(VFD_FRAME_VOLUME));
	vfd_cmd_P(PSTR(VFD_FRAME_LETTER));

	/* e-mail letter + inner red color */
	vfd_cmd_P(PSTR(VFD_SYMBOL_LETTER_INNER));
	vfd_cmd_P(PSTR(VFD_SYMBOL_LETTER_OUTER));

	/* volume area */
	vfd_cmd_P(PSTR(VFD_VOLUME_RED_LINE));
	vfd_cmd_P(PSTR(VFD_VOLUME_SMALL1));
	vfd_cmd_P(PSTR(VFD_VOLUME_SMALL2));
	vfd_cmd_P(PSTR(VFD_VOLUME_SMALL3));
	vfd_cmd_P(PSTR(VFD_UNMUTE));
	vfd_cmd_P(PSTR(VFD_MUTE));
	vfd_cmd_P(PSTR(VFD_VOLUME1));
	vfd_cmd_P(PSTR(VFD_VOLUME2));
	vfd_cmd_P(PSTR(VFD_VOLUME3));
	vfd_cmd_P(PSTR(VFD_VOLUME4));
	vfd_cmd_P(PSTR(VFD_VOLUME5));
	vfd_cmd_P(PSTR(VFD_VOLUME6));
	vfd_cmd_P(PSTR(VFD_VOLUME7));
	
	/* enable symbols 
	 * 
	 * these symbols support dimming. (..._DIMx)
	 * */
	vfd_cmd_P(PSTR(VFD_SYMBOL_1394_DIM3));
	vfd_cmd_P(PSTR(VFD_SYMBOL_CD_DIM3));
	vfd_cmd_P(PSTR(VFD_SYMBOL_HDD_DIM3));
	vfd_cmd_P(PSTR(VFD_SYMBOL_MOVIE_DIM3));
	vfd_cmd_P(PSTR(VFD_SYMBOL_MUSIC_DIM3));
	vfd_cmd_P(PSTR(VFD_SYMBOL_PICTURE_DIM3));
	vfd_cmd_P(PSTR(VFD_SYMBOL_USB_DIM3));
	vfd_cmd_P(PSTR(VFD_SYMBOL_TV_DIM3));
	
	/* red record button supports blinking
	 * cd/dvd symbol supports spinning
	 */
	vfd_cmd_P(PSTR(VFD_REC_BLINK1));
	vfd_cmd_P(PSTR(VFD_CD_ROTATE4));	
	
	delay(1000);
	
	/* show an animated space invader next to record button */
	for(;;) {
		vfd_cmd_P(PSTR(VFD_REC_GFXDATA_HEADER));
		vfd_cmd_P(PSTR("\x09\x00\x60\x1d\x1a\x2e\x1a\x1d\x60\x00"));
		_delay_ms(200);
		vfd_cmd_P(PSTR(VFD_REC_GFXDATA_HEADER));
		vfd_cmd_P(PSTR("\x09\x02\x01\x3d\x1a\x3e\x1a\x3d\x01\x02"));
		_delay_ms(200);
	}
	
	
}

/* _delay_ms only works with a constant argument. This wrapper
 * enables dynamic changing of delays.
 */
void delay(uint16_t ms) {
	while(ms) {
		_delay_ms(1);
		ms--;
	}
}

/* send string from RAM */
void uart_puts(char *str) {
	while(*str) {
		uart_putc(*str);
		str++;
	}
}

/* send character from RAM */
void uart_putc(char c) {
    while (!(UCSRA&(1<<UDRE)));
    UDR = c;
    return;
}

/* For this demo we don't need RX part of UART, for a real application
 * most likely you would enable these parts.
 */
void uart_init(void) {
    DDRD |=  (1<<PD1);		// TX Pin
    //DDRD &= ~(1<<PD0);	// RX Pin
    //PORTD |= (1<<PD0);
    
    uint16_t ubrr = (uint16_t)((uint32_t)F_CPU/(16*BAUDRATE)-1);
    UBRRH = (uint8_t)(ubrr>>8);
    UBRRL = (uint8_t)(ubrr);
 
    // 8N1, asynchron 
    UCSRB = (1<<TXEN);
    //UCSRB |= (1<<RXEN);				// also enable RX
    UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);

    // Flush Receive-Buffer  >> DO IF RX ENABLED <<
    //do {
        //UDR;
    //}
    //while (UCSRA&(1<<RXC));
}
